<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://codeneric.com
 * @since      1.0.0
 *
 * @package    Ultimate_Ads_Manager
 * @subpackage Ultimate_Ads_Manager/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Ultimate_Ads_Manager
 * @subpackage Ultimate_Ads_Manager/admin
 * @author     Codeneric <contact@codeneric.com>
 */

class Ultimate_Ads_Manager_Settings  {
     /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    private $display_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */

    private $plugin_options_key = 'codeneric_ad_options';
    private $general_settings_key;// = 'codeneric_ad_general_settings';
    public $general_settings;


    private $default_settings = array(
        'block_adblock' => 0
    );

    public function __construct($plugin_name,$version, $display_name ) {

        global $cc_uam_config;

        $this->general_settings_key = $cc_uam_config['general_settings_key'];

        $this->plugin_name = $plugin_name;
        $this->display_name = $display_name;
        $this->version = $version;

        $temp = get_option('timezone_string');
        $this->default_settings['block_timezone'] = empty($temp) ? 'America/New_York' : $temp;
        //echo "TZZzzzzzzzzzzzzzzzzzooooooooooooooooon:" . $this->default_settings['block_timezone'];
    }


    public function register_general_settings() {

        register_setting( $this->general_settings_key, $this->general_settings_key );
        $section_key = 'section_general';

        add_settings_section( $section_key, __('General Settings'), array( &$this, 'section_general_desc' ), $this->general_settings_key );

        
        add_settings_field( 'block_adblock', __('Block AdBlock'), array( &$this, 'field_block_adbock' ), $this->general_settings_key, $section_key );
        add_settings_field( 'GDPR_compliance', __('Enable GDPR compliance'), array( &$this, 'field_GDPR_compliance' ), $this->general_settings_key, $section_key );
        add_settings_field( 'ga_tracking_id', __('Google Analytics Tracking ID'), array( &$this, 'field_ga_tracking_id' ), $this->general_settings_key, $section_key );
        add_settings_field( 'db_optimization', __('Database Optimization'), array( &$this, 'db_optimization' ), $this->general_settings_key, $section_key );

        //add_settings_field( 'block_timezone', __('Your Timezone'), array( &$this, 'field_block_timezone' ), $this->general_settings_key, $section_key );

    }
    public function section_general_desc() { echo ''; }
    public function field_block_adbock() {
        ?>
        <label title="Block AdBlock">
            <input type="checkbox" name="<?php echo $this->general_settings_key; ?>[block_adblock]"  <?php checked( $this->general_settings['block_adblock'], 1 ); ?>  value="1" />
            <small><?php echo __('Success not guaranteed.'); ?></small>
        </label>



        <?php
    }

    public function field_GDPR_compliance() {
        ?>
        <label title="Enable GDPR compliance">
            <input type="checkbox" name="<?php echo $this->general_settings_key; ?>[GDPR_compliance]"  <?php checked( $this->general_settings['GDPR_compliance'], 1 ); ?>  value="1" />
            
        </label>



        <?php
    }

    public function field_ga_tracking_id() {
        $value = isset($this->general_settings['ga_tracking_id']) ? $this->general_settings['ga_tracking_id'] : '';
        ?>
            <label title="Google Analytics Tracking ID">
                <input type="text" name="<?php echo $this->general_settings_key; ?>[ga_tracking_id]"
                       placeholder="UA-xxxxxxxx-x"
                       value="<?php echo $value ?>"/>
            </label>
            <?php

        }

    public function field_block_timezone(){
        $tz = DateTimeZone::listIdentifiers();

        ?>
        <label title="Your Timezone">
            <select name="<?php echo $this->general_settings_key; ?>[block_timezone]"
                    value="" >
                <?php foreach($tz as $key=>$value): ?>
                    <option value="<?php echo $value; ?>" <?php
                    if(isset($this->general_settings['block_timezone']) && $this->general_settings['block_timezone'] === $value)
                        echo 'selected';
                    ?>><?php echo $value; ?></option>
                <?php endforeach; ?>
            </select>
            <button class="button" id="guess-tz">guess timezone</button>
            <small><?php echo __('Click this button if you want to automatically guess your timezone by your local time.') ?></small>

        </label>

        <?php
    }

    public function db_optimization_OUT(){
        global $cc_uam_config;
        $value = isset($this->general_settings['db_optimization_enabled']) ? $this->general_settings['db_optimization_enabled'] : false;

        ?>
        <label title="Database Optimization">
<!--            --><?php //foreach ($cc_uam_config['forgettable_properties'] as $prop){ ?>
<!--            <label title="--><?php //echo $prop; ?><!--" ></label>-->
<!--                <input type="checkbox" value="--><?php //echo $prop; ?><!--" name="--><?php //echo $this->general_settings_key; ?><!--[dp_optimization][forget][]"/>-->
<!--            --><?php //} ?>
        </label>
        <?php
    }

    public function db_optimization() {
        ?>
        <label title="Block AdBlock">
            <input type="checkbox" name="<?php echo $this->general_settings_key; ?>[block_adblock]"  <?php checked( $this->general_settings['block_adblock'], 1 ); ?>  value="1" />
            <small><?php echo __('Success not guaranteed.'); ?></small>
        </label>



        <?php
    }

    public function load_settings() {
        $this->general_settings = (array)  get_option( $this->general_settings_key );
        $this->general_settings = array_merge( $this->default_settings, $this->general_settings );

    }
}

