<?php
/**
 * Class SampleTest
 *
 * @package Ultimate_Ads_Manager
 */

/**
 * Sample test case.
 */
class StatisticsTest extends WP_UnitTestCase {

	/**
	 * A single example test.
	 */


	function mock_map($name){
	    $m = array(
	        'test_24h' => '1-day',
	        'test_all_time' => '1-year',
            );
	    return $m[$name];
    }
	function setUp(){
        mock_statistics($this->mock_map($this->getName()));

    }
	function test_24h() {
	    $stats = new Statistics_Calculator();

        $time_a = strtotime('2017-01-01 00:00:00');
        $time_b = strtotime('2017-01-01 23:59:59');

        $periods_1_v_t = $stats->get_periods(1, 'view','total', 42,
            $time_a, $time_b,'hour', null );

        $expected_total_views_1 = array(3, 5, 8, 3, 9, 7, 7, 8, 5, 8, 7, 2, 2, 10, 5, 4, 10, 6, 9, 6, 6, 5, 6, 8);

        $this->assertEquals(24, count($periods_1_v_t));
        foreach ($periods_1_v_t as $i => $period){
            $this->assertEquals($expected_total_views_1[$i], $period->data);
        }

        $periods_1_c_t = $stats->get_periods(1, 'click','total', 42,
            $time_a, $time_b,'hour', null );
        $this->assertEquals(24, count($periods_1_c_t));

        $periods_2_v_t = $stats->get_periods(2, 'view','total', 42,
            $time_a, $time_b,'hour', null );
        $this->assertEquals(24, count($periods_2_v_t));

        $periods_2_c_t = $stats->get_periods(2, 'click','total', 42,
            $time_a, $time_b,'hour', null );
        $this->assertEquals(24, count($periods_2_c_t));

        $sum = 0;
        $periods = array($periods_1_v_t, $periods_1_c_t, $periods_2_v_t, $periods_2_c_t);

        foreach ($periods as $period){
            foreach ($period as $hour)
                $sum += $hour->data;
        }

        $this->assertEquals(500, $sum);
    }

    function test_all_time() {
        $stats = new Statistics_Calculator();

        $time_a = strtotime('2017-01-01 00:00:00');
        $time_b = strtotime('2017-12-31 23:59:59');

        $periods_1_v_t = $stats->get_periods(1, 'view','total', 42,
            $time_a, $time_b,'infinite', null );

        $this->assertEquals(1, count($periods_1_v_t));
        $this->assertEquals(253, $periods_1_v_t[0]->data);


        $periods_1_c_t = $stats->get_periods(1, 'click','total', 42,
            $time_a, $time_b,'infinite', null );
        $this->assertEquals(1, count($periods_1_c_t));

        $periods_2_v_t = $stats->get_periods(2, 'view','total', 42,
            $time_a, $time_b,'infinite', null );
        $this->assertEquals(1, count($periods_2_v_t));

        $periods_2_c_t = $stats->get_periods(2, 'click','total', 42,
            $time_a, $time_b,'infinite', null );
        $this->assertEquals(1, count($periods_2_c_t));

        $sum = 0;
        $periods = array($periods_1_v_t, $periods_1_c_t, $periods_2_v_t, $periods_2_c_t);

        foreach ($periods as $period){
            foreach ($period as $hour)
                $sum += $hour->data;
        }

        $this->assertEquals(1000, $sum);
    }



	function tearDown()
    {
        parent::tearDown(); // TODO: Change the autogenerated stub
        drop_events_table();

    }
}
